﻿#region Using Statements
using System;
using System.Collections.Generic;
using RhinoXNA;
using RhinoXNA.RhinoGraphics.Rhino2D;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework;
using RhinoXNA.RhinoUtilities;
using RhinoXNA.RhinoGameObjects;
using RhinoXNA.RhinoGraphics.Rhino2D.RhinoHUD;
using AsteroidsGame.Base_Objects;
using AsteroidsGame.Menus;
using AsteroidsGame.Utilities;
#endregion

namespace AsteroidsGame.Game_Objects
{
    /// <summary>
    /// Class defining player avatar (ship)
    /// </summary>
    public class Ship : SpriteEntity
    {
        #region Private Variables
        // Movement variables
        private Vector2 _movementSpeed = new Vector2(5.0f, 5.0f);
        private Vector2 _movementDir = new Vector2(0.0f, -1.0f);

        // Variables for ship reset (after death)
        private bool _isResetting = true;
        private float _resetTime = 0.0f;
        private float _blinkTime = 0.0f;

        // Life counter
        private int _livesLeft = 3;

        // Class input binding
        private InputBinding _input;
        #endregion

        #region Public Methods
        /// <summary>
        /// Constructs a new instance of this class type
        /// </summary>
        /// <param name="parameters">Parameters defining the new object</param>
        public Ship(List<String> parameters)
            : base(parameters)
        {
            _input = new InputBinding(Keyboard.GetState(), Mouse.GetState());
            DrawLives();
        }
        #endregion

        #region Private Methods
        /// <summary>
        /// Draws the current number of lives on the screen (removes previous font entity)
        /// </summary>
        private void DrawLives()
        {
            RhinoHUDManager.Instance.RemoveEntity("LivesCount");

            RhinoHUDManager.Instance.AddEntity(new HUDText(new List<String>()
            {
                "HUDLives", 
                "Fonts\\HUDFont", 
                "LivesCount", 
                _livesLeft.ToString(), 
                "L240", "T50", 
                "0", 
                "1.0", "1.0", 
                "0.6", 
                "true"
            }));
        }

        /// <summary>
        /// Setes the ships position (on death)
        /// </summary>
        private void ResetShip()
        {
            _isResetting = true;
            this.Position = new Vector2(RhinoXNAAdapter.Instance.SCREEN_SAFE.Center.X,
                RhinoXNAAdapter.Instance.SCREEN_SAFE.Center.Y);
        }

        /// <summary>
        /// Blinks a ship that has been reset
        /// </summary>
        private void BlinkResettingShip(GameTime gameTime)
        {
            // If resetting is true, start/continue blinking
            if (_isResetting)
            {
                _resetTime += gameTime.ElapsedGameTime.Milliseconds;
                _blinkTime += gameTime.ElapsedGameTime.Milliseconds;

                if (_blinkTime >= 200.0f)
                {
                    Ship entity = (Ship)RhinoEntityManager.Instance.Entities.Find(delegate(IRhinoGraphicsComponent rg) { return rg.Handle == this.Handle; });
                    entity.isVisible = !entity.isVisible;
                    _blinkTime = 0.0f;
                }

                if (_resetTime >= 2000.0f)
                {
                    _isResetting = false;
                    _resetTime = 0.0f;

                    // Make sure you're visible!
                    Ship entity = (Ship)RhinoEntityManager.Instance.Entities.Find(delegate(IRhinoGraphicsComponent rg) { return rg.Handle == this.Handle; });
                    entity.isVisible = true;
                }
            }
        }

        /// <summary>
        /// Checks and alters positioning data for the ship
        /// </summary>
        /// <param name="kbs">Current keyboard state</param>
        private void UpdatePositionData(KeyboardState kbs)
        {
            if (kbs.IsKeyDown(Keys.W))
            {
                this.Position = new Microsoft.Xna.Framework.Vector2(this.Position.X + (_movementDir.X * _movementSpeed.X),
                    this.Position.Y + (_movementDir.Y * _movementSpeed.Y));
            }

            if (kbs.IsKeyDown(Keys.S))
            {
                this.Position = new Microsoft.Xna.Framework.Vector2(this.Position.X - (_movementDir.X * _movementSpeed.X),
                    this.Position.Y - (_movementDir.Y * _movementSpeed.Y));
            }
        }

        /// <summary>
        /// Checks and alters rotation data for the ship
        /// </summary>
        /// <param name="kbs">Current keyboard state</param>
        private void UpdateRotationData(KeyboardState kbs)
        {
            if (kbs.IsKeyDown(Keys.A))
            {
                this.Rotation -= MathHelper.ToRadians(5);
                float translatedRot = MathHelper.ToRadians(MathHelper.ToDegrees(this.Rotation) + 240);

                Matrix transform = new Matrix();
                transform = Matrix.CreateTranslation(new Vector3(this.Position, 1.0f))
                    * Matrix.CreateRotationZ(translatedRot);

                _movementDir = Vector2.Transform(_movementDir, transform);
                _movementDir.Normalize();
            }

            if (kbs.IsKeyDown(Keys.D))
            {
                this.Rotation += MathHelper.ToRadians(5);
                float translatedRot = MathHelper.ToRadians(MathHelper.ToDegrees(this.Rotation) + 240);

                Matrix transform = new Matrix();
                transform = Matrix.CreateTranslation(new Vector3(this.Position, 1.0f))
                    * Matrix.CreateRotationZ(translatedRot);

                _movementDir = Vector2.Transform(_movementDir, transform);
                _movementDir.Normalize();
            }
        }
        #endregion

        #region Protected Methods
        /// <summary>
        /// Overridden implementation of Animate (called when updating)
        /// </summary>
        /// <param name="gameTime">Current gametime</param>
        protected override void Animate(GameTime gameTime)
        {
            // Variable to grab user input
            KeyboardState kbs = Keyboard.GetState();

            BlinkResettingShip(gameTime);

            UpdatePositionData(kbs);

            Vector2 pos = this.Position;
            ScreenAreaUtility.WrapScreen(ref pos, this.Dimensions);
            this.Position = pos;

            UpdateRotationData(kbs);

            // Check for fire condition and create new bullet
            if (_input.CheckKeyDown(kbs, Keys.Space))
            {
                List<String> parameters = new List<String>()
                {
                    "Bullet",
                    "Sprites\\shot",
                    "bullet" + RhinoEntityManager.Instance.Entities.Count.ToString(), 
                    this.Position.X.ToString(), this.Position.Y.ToString(),
                    Convert.ToInt32(MathHelper.ToDegrees(this.Rotation)).ToString(), 
                    "1.0", "1.0", 
                    "0.7", "true", 
                    "0", "0", "WIDTH", "HEIGHT",
                    _movementDir.X.ToString(), _movementDir.Y.ToString()
                };
                RhinoEntityManager.Instance.AddEntity(new Bullet(parameters));

                GameAudioEngine.Instance.PlayShotSound();
            }

            // Check for collisions if not resetting
            if (!_isResetting)
            {
                for (int i = 0; i < AsteroidFactory.Instance.Asteroids.Count; i++)
                {
                    if (this.BasicBoundingBox.Intersects(AsteroidFactory.Instance.Asteroids[i].BasicBoundingBox))
                    {
                        _livesLeft--;
                        DrawLives();

                        List<String> parameters = new List<String>()
                    {
                        "Explosion",
                        "Sprites\\explosion",
                        "explosion", 
                        this.Position.X.ToString(), this.Position.Y.ToString(),
                        Convert.ToInt32(MathHelper.ToDegrees(this.Rotation)).ToString(), 
                        "1.0", "1.0", 
                        "0.7", "true", 
                        "0", "0", "WIDTH", "HEIGHT"
                    };
                        RhinoEntityManager.Instance.AddEntity(new Explosion(parameters));

                        GameAudioEngine.Instance.PlayExplosionSound();

                        ResetShip();
                    }
                }

                if (this._livesLeft <= 0)
                {
                    RhinoXNAAdapter.Instance.AddNewMenu(new GameOverMenu());
                }
            }

            _input.Update();
        }
        #endregion
    }
}
