﻿#region Using Statements
using System;
using System.Collections.Generic;
#endregion

namespace RhinoXNA.RhinoGameObjects
{
    /// <summary>
    /// Stores all drawable components (non-HUD)
    /// </summary>
    public sealed class RhinoEntityManager
    {
        #region Private Variables
        // Stored entity list
        private static List<IRhinoGraphicsComponent> _entities;

        // Singleton stuff
        private static volatile RhinoEntityManager _manager;
        private static object syncRoot = new Object();
        #endregion

        #region Public Properties
        /// <summary>
        /// Returns the current list of stored entities
        /// </summary>
        public List<IRhinoGraphicsComponent> Entities { get { return _entities; } }

        /// <summary>
        /// Returns the global instance of a RhinoEntityManager
        /// </summary>
        public static RhinoEntityManager Instance
        {
            get
            {
                if (_manager == null)
                {
                    lock (syncRoot)
                    {
                        if (_manager == null)
                        {
                            _manager = new RhinoEntityManager();
                        }
                    }
                }

                return _manager;
            }
        }
        #endregion

        #region Private Methods
        /// <summary>
        /// Creates a new RhinoEntityManager
        /// </summary>
        private RhinoEntityManager()
        {
            _entities = new List<IRhinoGraphicsComponent>();
        }
        #endregion

        #region Public Methods
        /// <summary>
        /// Adds a new entity to the list of stored entities
        /// </summary>
        /// <param name="entity">Entity to store</param>
        public void AddEntity(IRhinoGraphicsComponent entity)
        {
            _entities.Add(entity);
        }

        /// <summary>
        /// Adds a list of entities to the stored entities list
        /// </summary>
        /// <param name="entities">List of entities to store</param>
        public void AddEntities(List<IRhinoGraphicsComponent> entities)
        {
            foreach (IRhinoGraphicsComponent entity in entities)
            {
                _entities.Add(entity);
            }
        }

        /// <summary>
        /// Clears the stored entity list
        /// </summary>
        public void Clear()
        {
            _entities.Clear();
        }

        /// <summary>
        /// Removes all entities with the given handle (name)
        /// </summary>
        /// <param name="entityHandle">Name of entity to remove</param>
        public void RemoveEntity(String entityHandle)
        {
            _entities.RemoveAll(delegate(IRhinoGraphicsComponent ir) { return ir.Handle == entityHandle; });
        }
        #endregion
    }
}
