﻿#region Using Statements
using System;
using System.IO;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Content;
using System.Collections;
using RhinoXNA.RhinoGraphics.Rhino2D;
#endregion

namespace RhinoXNA.RhinoUtilities
{
    /// <summary>
    /// Utility class used to load graphics and parse files
    /// </summary>
    public class RhinoLoader
    {
        #region Public Methods
        /// <summary>
        /// Returns a list of IRhinoGraphicsComponent objects loaded from a file
        /// </summary>
        /// <param name="propertiesFile">Properties file linking classes</param>
        /// <param name="fileToLoad">File to parse and load objects from</param>
        /// <returns>List of IRhinoGraphicsComponent based objects of the specified types</returns>
        public static List<IRhinoGraphicsComponent> GetGraphics(String propertiesFile, String fileToLoad)
        {
            return LoadGraphics(LoadProperties(propertiesFile), fileToLoad);
        }

        /// <summary>
        /// Returns a List of IMenuItem objects loaded from the given file
        /// </summary>
        /// <param name="propertiesFile">Properties file linking classes</param>
        /// <param name="fileToLoad">File to parse and create objects from</param>
        /// <returns>List of IRhinoGraphicsComponent based objects of the specified types</returns>
        public static List<IMenuItem> GetMenuItems(String propertiesFile, String fileToLoad)
        {
            return LoadMenuItems(LoadProperties(propertiesFile), fileToLoad);
        }

        /// <summary>
        /// Returns the Position of the center of a texture given string literals
        /// </summary>
        /// <param name="texture">The texture to load</param>
        /// <param name="xPos">X-Position string literal</param>
        /// <param name="yPos">Y-Position string literal</param>
        /// <returns>Vector2 defining the center of the texture's on-screen position</returns>
        public static Vector2 GetPosition(Texture2D texture, String xPos, String yPos)
        {
            Vector2 pos = new Vector2();

            //X-Position
            if (xPos.ToLower() == "center")
                pos.X = RhinoXNAAdapter.Instance.SCREEN_SAFE.Center.X;
            else if (xPos.StartsWith("C"))
                pos.X = RhinoXNAAdapter.Instance.SCREEN_SAFE.Center.X + float.Parse(xPos.Substring(1));
            else if (xPos.StartsWith("L"))
                pos.X = RhinoXNAAdapter.Instance.SCREEN_SAFE.Left + float.Parse(xPos.Substring(1));
            else if (xPos.StartsWith("R"))
                pos.X = RhinoXNAAdapter.Instance.SCREEN_SAFE.Right - float.Parse(xPos.Substring(1));
            else
                pos.X = float.Parse(xPos);

            //Y-Position
            if (yPos.ToLower() == "center")
                pos.Y = RhinoXNAAdapter.Instance.SCREEN_SAFE.Center.Y;
            else if (yPos.StartsWith("C"))
                pos.Y = RhinoXNAAdapter.Instance.SCREEN_SAFE.Center.Y + float.Parse(yPos.Substring(1));
            else if (yPos.StartsWith("B"))
                pos.Y = RhinoXNAAdapter.Instance.SCREEN_SAFE.Bottom - float.Parse(yPos.Substring(1));
            else if (yPos.StartsWith("T"))
            {
                if (yPos.StartsWith("TBG"))
                    pos.Y = texture.Height / 2 + float.Parse(yPos.Substring(3));
                else
                    pos.Y = RhinoXNAAdapter.Instance.SCREEN_SAFE.Top + float.Parse(yPos.Substring(1));
            }
            else
            {
                pos.Y = float.Parse(yPos);
            }

            return pos;
        }

        /// <summary>
        /// Returns the on-screen position of a specific set of string literals
        /// </summary>
        /// <param name="xPos">X-Position string literal</param>
        /// <param name="yPos">Y-Position string literal</param>
        /// <returns>Vector2 on-screen position generated from the string literal arguments</returns>
        public static Vector2 GetPosition(String xPos, String yPos)
        {
            Vector2 pos = new Vector2();

            //X-Position
            if (xPos.ToLower() == "center")
                pos.X = RhinoXNAAdapter.Instance.SCREEN_SAFE.Center.X;
            else if (xPos.StartsWith("C"))
                pos.X = RhinoXNAAdapter.Instance.SCREEN_SAFE.Center.X + float.Parse(xPos.Substring(1));
            else if (xPos.StartsWith("L"))
                pos.X = RhinoXNAAdapter.Instance.SCREEN_SAFE.Left + float.Parse(xPos.Substring(1));
            else if (xPos.StartsWith("R"))
                pos.X = RhinoXNAAdapter.Instance.SCREEN_SAFE.Right - float.Parse(xPos.Substring(1));
            else
                pos.X = float.Parse(xPos);

            //Y-Position
            if (yPos.ToLower() == "center")
                pos.Y = RhinoXNAAdapter.Instance.SCREEN_SAFE.Center.Y;
            else if (yPos.StartsWith("C"))
                pos.Y = RhinoXNAAdapter.Instance.SCREEN_SAFE.Center.Y + float.Parse(yPos.Substring(1));
            else if (yPos.StartsWith("B"))
                pos.Y = RhinoXNAAdapter.Instance.SCREEN_SAFE.Bottom - float.Parse(yPos.Substring(1));
            else if (yPos.StartsWith("T"))
                pos.Y = RhinoXNAAdapter.Instance.SCREEN_SAFE.Top + float.Parse(yPos.Substring(1));
            else
                pos.Y = float.Parse(yPos);

            return pos;
        }

        /// <summary>
        /// Returns the basic source rectangle size given string literals
        /// </summary>
        /// <param name="texture">Texture to load from</param>
        /// <param name="width">Width string literal</param>
        /// <param name="height">Height string literal</param>
        /// <returns>Vector2 defining the width and height of the source rectangle</returns>
        public static Vector2 GetSourceRectSize(Texture2D texture, String width, String height)
        {
            Vector2 boundingSize = new Vector2(0.0f, 0.0f);

            //Width
            if (width.ToLower() == "width")
                boundingSize.X = texture.Width;
            else
                boundingSize.X = float.Parse(width);

            //Height
            if (height.ToLower() == "height")
                boundingSize.Y = texture.Height;
            else
                boundingSize.Y = float.Parse(height);

            return boundingSize;
        }
        #endregion

        #region Private Methods
        /// <summary>
        /// Returns a list of IRhinoGraphicsComponent objects parsed from a file
        /// </summary>
        /// <param name="types">Class types the object could be</param>
        /// <param name="fileToLoad">File to parse and load objects from</param>
        /// <returns>List of IRhinoGraphicsComponent based objects of the specified types</returns>
        private static List<IRhinoGraphicsComponent> LoadGraphics(Hashtable types, String fileToLoad)
        {
            List<IRhinoGraphicsComponent> graphics = new List<IRhinoGraphicsComponent>();

            try
            {
                using (StreamReader sr = new StreamReader(StorageContainer.TitleLocation + "\\Content\\" + fileToLoad))
                {
                    String line;

                    // Read file
                    while ((line = sr.ReadLine()) != null)
                    {
                        String[] temp = getParameters(line);

                        // If not comment, parse
                        if (temp[0].StartsWith("'") != true && line.Replace(" ", "") != "")
                        {
                            String objTypeString = types[temp[0]].ToString();

                            // Setup object of type
                            System.Reflection.Assembly assembly = System.Reflection.Assembly.LoadFile(Directory.GetCurrentDirectory()
                                + "\\"
                                + System.Reflection.Assembly.GetEntryAssembly().FullName.Substring(0, System.Reflection.Assembly.GetEntryAssembly().FullName.IndexOf(","))
                                + ".exe");
                            Type classType = assembly.GetType(objTypeString);

                            if (classType != null)
                            {
                                // Create object of type
                                List<String> parameters = new List<string>(temp);
                                object obj = Activator.CreateInstance(classType, new object[] { parameters });
                                graphics.Add(obj as IRhinoGraphicsComponent);
                            }
                        }
                    }
                }
            }
            catch (Exception e)
            {
                //QUIT!
                throw new Exception("RhinoXNA.RhinoUtilities.RhinoLoader Error: Error Parsing Load File - Shutting Down");
            }

            return graphics;
        }

        /// <summary>
        /// Returns a List of IMenuItem objects loaded from the given file
        /// </summary>
        /// <param name="types">Types of objects that can be created</param>
        /// <param name="fileToLoad">File to parse and create objects from</param>
        /// <returns>List of IMenuItem objects of the given types</returns>
        private static List<IMenuItem> LoadMenuItems(Hashtable types, String fileToLoad)
        {
            List<IMenuItem> graphics = new List<IMenuItem>();

            try
            {
                using (StreamReader sr = new StreamReader(StorageContainer.TitleLocation + "\\Content\\" + fileToLoad))
                {
                    String line;

                    // Read through file
                    while ((line = sr.ReadLine()) != null)
                    {
                        String[] temp = getParameters(line);

                        // If not comment, parse
                        if (temp[0].StartsWith("'") != true && line.Replace(" ", "") != "")
                        {
                            //Setup new object
                            String objTypeString = types[temp[0]].ToString();
                            System.Reflection.Assembly assembly = System.Reflection.Assembly.LoadFile(Directory.GetCurrentDirectory()
                                + "\\"
                                + System.Reflection.Assembly.GetEntryAssembly().FullName.Substring(0, System.Reflection.Assembly.GetEntryAssembly().FullName.IndexOf(","))
                                + ".exe");
                            Type classType = assembly.GetType(objTypeString);

                            if (classType != null)
                            {
                                //Instantiate new object of found type
                                List<String> parameters = new List<string>(temp);
                                object obj = Activator.CreateInstance(classType, new object[] { parameters });
                                graphics.Add(obj as IMenuItem);
                            }
                        }
                    }
                }
            }
            catch (Exception e)
            {
                //QUIT!
                throw new Exception("RhinoXNA.RhinoUtilies.RhinoLoader Error: Error Parsing Load File - Shutting Down");
            }

            return graphics;
        }

        /// <summary>
        /// Returns a set of string-object type key pairs
        /// </summary>
        /// <param name="propertiesFile">Properties file to load from</param>
        /// <returns>Hashtable of string-object type pairs</returns>
        private static Hashtable LoadProperties(String propertiesFile)
        {
            Hashtable properties = new Hashtable();

            try
            {
                // Read file
                using (StreamReader sr = new StreamReader(StorageContainer.TitleLocation + "\\Content\\" + propertiesFile))
                {
                    String line;

                    // Read lines
                    while ((line = sr.ReadLine()) != null)
                    {
                        line = line.Replace(" ", "");
                        String[] temp = line.Split('=');

                        // If not comment, parse
                        if (temp[0].StartsWith("'") != true && line.Replace(" ", "") != "")
                        {
                            // Add new type match
                            properties.Add(temp[0], temp[1]);
                        }
                    }
                }
            }
            catch (Exception e)
            {
                //QUIT!
                throw new Exception("Error Parsing Properties File - Shutting Down");
            }

            return properties;
        }

        /// <summary>
        /// Returns a set of parameters given a text line
        /// </summary>
        /// <param name="line">Line of text to parse from</param>
        /// <returns>String array of parameters</returns>
        private static String[] getParameters(String line)
        {
            String[] temp = line.Split(',');

            for (int i = 0; i < temp.Length; i++)
            {
                if (temp[i].Contains("\""))
                {
                    temp[i] = temp[i].Replace("\"", "");
                }
                else
                {
                    temp[i] = temp[i].Replace(" ", "");
                }
            }

            return temp;
        }
        #endregion
    }
}